# coding: utf-8

"""
    Asteroid Agents API

    Version 1 of the Asteroid Agents API

    The version of the OpenAPI document: v1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class CountryCode(str, Enum):
    """
    Two-letter country code for proxy location
    """

    """
    allowed enum values
    """
    US = 'us'
    UK = 'uk'
    FR = 'fr'
    IT = 'it'
    JP = 'jp'
    AU = 'au'
    DE = 'de'
    FI = 'fi'
    CA = 'ca'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of CountryCode from a JSON string"""
        return cls(json.loads(json_str))





from .client import (
    AsteroidClient,
    create_client,
    execute_agent,
    get_execution_status,
    get_execution_result,
    wait_for_execution_result,
    upload_execution_files,
    get_browser_session_recording,
    get_agent_profiles,
    get_agent_profile,
    create_agent_profile,
    update_agent_profile,
    delete_agent_profile,
    get_last_n_execution_activities,
    add_message_to_execution,
    wait_for_agent_interaction,
    get_credentials_public_key,
    AsteroidAPIError,
    ExecutionError,
    TimeoutError,
    AgentInteractionResult
)
from .agents_v1_gen import ExecutionResult

__all__ = [
    'AsteroidClient',
    'create_client',
    'execute_agent',
    'get_execution_status',
    'get_execution_result',
    'wait_for_execution_result',
    'upload_execution_files',
    'get_browser_session_recording',
    'get_agent_profiles',
    'get_agent_profile',
    'create_agent_profile',
    'update_agent_profile',
    'delete_agent_profile',
    'get_last_n_execution_activities',
    'add_message_to_execution',
    'wait_for_agent_interaction',
    'get_credentials_public_key',
    'AsteroidAPIError',
    'ExecutionError',
    'TimeoutError',
    'AgentInteractionResult',
    'ExecutionResult'
]



# coding: utf-8

"""
    Asteroid Agents API

    Version 1 of the Asteroid Agents API

    The version of the OpenAPI document: v1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBytes, StrictStr
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated
from asteroid_odyssey.agents_v1_gen.models.browser_session_recording_response import BrowserSessionRecordingResponse
from asteroid_odyssey.agents_v1_gen.models.execution_response import ExecutionResponse
from asteroid_odyssey.agents_v1_gen.models.execution_result_response import ExecutionResultResponse
from asteroid_odyssey.agents_v1_gen.models.execution_status_response import ExecutionStatusResponse
from asteroid_odyssey.agents_v1_gen.models.structured_agent_execution_request import StructuredAgentExecutionRequest
from asteroid_odyssey.agents_v1_gen.models.upload_execution_files200_response import UploadExecutionFiles200Response

from asteroid_odyssey.agents_v1_gen.api_client import ApiClient, RequestSerialized
from asteroid_odyssey.agents_v1_gen.api_response import ApiResponse
from asteroid_odyssey.agents_v1_gen.rest import RESTResponseType


class ExecutionApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def execute_agent(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the agent")],
        request_body: Dict[str, Any],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ExecutionResponse:
        """(Deprecated) Execute an agent

        Executes an agent with the provided parameters

        :param id: The ID of the agent (required)
        :type id: str
        :param request_body: (required)
        :type request_body: Dict[str, object]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501
        warnings.warn("POST /agent/{id} is deprecated.", DeprecationWarning)

        _param = self._execute_agent_serialize(
            id=id,
            request_body=request_body,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '202': "ExecutionResponse",
            '400': "ErrorResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def execute_agent_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the agent")],
        request_body: Dict[str, Any],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ExecutionResponse]:
        """(Deprecated) Execute an agent

        Executes an agent with the provided parameters

        :param id: The ID of the agent (required)
        :type id: str
        :param request_body: (required)
        :type request_body: Dict[str, object]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501
        warnings.warn("POST /agent/{id} is deprecated.", DeprecationWarning)

        _param = self._execute_agent_serialize(
            id=id,
            request_body=request_body,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '202': "ExecutionResponse",
            '400': "ErrorResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def execute_agent_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the agent")],
        request_body: Dict[str, Any],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """(Deprecated) Execute an agent

        Executes an agent with the provided parameters

        :param id: The ID of the agent (required)
        :type id: str
        :param request_body: (required)
        :type request_body: Dict[str, object]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501
        warnings.warn("POST /agent/{id} is deprecated.", DeprecationWarning)

        _param = self._execute_agent_serialize(
            id=id,
            request_body=request_body,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '202': "ExecutionResponse",
            '400': "ErrorResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _execute_agent_serialize(
        self,
        id,
        request_body,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if request_body is not None:
            _body_params = request_body


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/agent/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def execute_agent_structured(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the agent")],
        structured_agent_execution_request: StructuredAgentExecutionRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ExecutionResponse:
        """Execute an agent with structured parameters

        Executes an agent with structured parameters including optional agent profile configuration. This is the recommended method for new integrations.

        :param id: The ID of the agent (required)
        :type id: str
        :param structured_agent_execution_request: (required)
        :type structured_agent_execution_request: StructuredAgentExecutionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._execute_agent_structured_serialize(
            id=id,
            structured_agent_execution_request=structured_agent_execution_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '202': "ExecutionResponse",
            '400': "ErrorResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def execute_agent_structured_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the agent")],
        structured_agent_execution_request: StructuredAgentExecutionRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ExecutionResponse]:
        """Execute an agent with structured parameters

        Executes an agent with structured parameters including optional agent profile configuration. This is the recommended method for new integrations.

        :param id: The ID of the agent (required)
        :type id: str
        :param structured_agent_execution_request: (required)
        :type structured_agent_execution_request: StructuredAgentExecutionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._execute_agent_structured_serialize(
            id=id,
            structured_agent_execution_request=structured_agent_execution_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '202': "ExecutionResponse",
            '400': "ErrorResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def execute_agent_structured_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the agent")],
        structured_agent_execution_request: StructuredAgentExecutionRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Execute an agent with structured parameters

        Executes an agent with structured parameters including optional agent profile configuration. This is the recommended method for new integrations.

        :param id: The ID of the agent (required)
        :type id: str
        :param structured_agent_execution_request: (required)
        :type structured_agent_execution_request: StructuredAgentExecutionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._execute_agent_structured_serialize(
            id=id,
            structured_agent_execution_request=structured_agent_execution_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '202': "ExecutionResponse",
            '400': "ErrorResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _execute_agent_structured_serialize(
        self,
        id,
        structured_agent_execution_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if structured_agent_execution_request is not None:
            _body_params = structured_agent_execution_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/agent/{id}/execute',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_browser_session_recording(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> BrowserSessionRecordingResponse:
        """Get browser session recording

        Retrieves the browser session recording URL for a completed execution

        :param id: The ID of the execution (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_browser_session_recording_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BrowserSessionRecordingResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_browser_session_recording_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[BrowserSessionRecordingResponse]:
        """Get browser session recording

        Retrieves the browser session recording URL for a completed execution

        :param id: The ID of the execution (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_browser_session_recording_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BrowserSessionRecordingResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_browser_session_recording_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get browser session recording

        Retrieves the browser session recording URL for a completed execution

        :param id: The ID of the execution (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_browser_session_recording_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "BrowserSessionRecordingResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_browser_session_recording_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/execution/{id}/browser_session/recording',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_execution_result(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ExecutionResultResponse:
        """Get execution result


        :param id: The ID of the execution (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_execution_result_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ExecutionResultResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_execution_result_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ExecutionResultResponse]:
        """Get execution result


        :param id: The ID of the execution (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_execution_result_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ExecutionResultResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_execution_result_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get execution result


        :param id: The ID of the execution (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_execution_result_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ExecutionResultResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_execution_result_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/execution/{id}/result',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_execution_status(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ExecutionStatusResponse:
        """Get execution status


        :param id: The ID of the execution (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_execution_status_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ExecutionStatusResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_execution_status_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ExecutionStatusResponse]:
        """Get execution status


        :param id: The ID of the execution (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_execution_status_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ExecutionStatusResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_execution_status_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get execution status


        :param id: The ID of the execution (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_execution_status_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ExecutionStatusResponse",
            '404': "ErrorResponse",
            '500': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_execution_status_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/execution/{id}/status',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def upload_execution_files(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        files: Annotated[Optional[List[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]]], Field(description="Files to upload to the execution")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UploadExecutionFiles200Response:
        """Upload files to an execution


        :param id: The ID of the execution (required)
        :type id: str
        :param files: Files to upload to the execution
        :type files: List[bytearray]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._upload_execution_files_serialize(
            id=id,
            files=files,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UploadExecutionFiles200Response",
            '400': "ErrorResponse",
            '401': "ErrorResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def upload_execution_files_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        files: Annotated[Optional[List[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]]], Field(description="Files to upload to the execution")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UploadExecutionFiles200Response]:
        """Upload files to an execution


        :param id: The ID of the execution (required)
        :type id: str
        :param files: Files to upload to the execution
        :type files: List[bytearray]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._upload_execution_files_serialize(
            id=id,
            files=files,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UploadExecutionFiles200Response",
            '400': "ErrorResponse",
            '401': "ErrorResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def upload_execution_files_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The ID of the execution")],
        files: Annotated[Optional[List[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]]], Field(description="Files to upload to the execution")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Upload files to an execution


        :param id: The ID of the execution (required)
        :type id: str
        :param files: Files to upload to the execution
        :type files: List[bytearray]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._upload_execution_files_serialize(
            id=id,
            files=files,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UploadExecutionFiles200Response",
            '400': "ErrorResponse",
            '401': "ErrorResponse",
            '404': "ErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _upload_execution_files_serialize(
        self,
        id,
        files,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'files': 'csv',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        if files is not None:
            _files['files'] = files
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'multipart/form-data'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/execution/{id}/files',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    Asteroid Agents API

    Version 1 of the Asteroid Agents API

    The version of the OpenAPI document: v1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from asteroid_odyssey.agents_v1_gen.models.cookie import Cookie
from asteroid_odyssey.agents_v1_gen.models.country_code import CountryCode
from asteroid_odyssey.agents_v1_gen.models.credential import Credential
from asteroid_odyssey.agents_v1_gen.models.proxy_type import ProxyType
from typing import Optional, Set
from typing_extensions import Self

class UpdateAgentProfileRequest(BaseModel):
    """
    UpdateAgentProfileRequest
    """ # noqa: E501
    name: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=255)]] = Field(default=None, description="The name of the agent profile")
    description: Optional[Annotated[str, Field(strict=True, max_length=1000)]] = Field(default=None, description="The description of the agent profile")
    proxy_cc: Optional[CountryCode] = None
    proxy_type: Optional[ProxyType] = None
    captcha_solver_active: Optional[StrictBool] = Field(default=None, description="Whether the captcha solver should be active for this profile")
    sticky_ip: Optional[StrictBool] = Field(default=None, description="Whether the same IP address should be used for all executions of this profile")
    credentials_to_add: Optional[List[Credential]] = Field(default=None, description="List of credentials to add to the profile")
    credentials_to_delete: Optional[List[StrictStr]] = Field(default=None, description="List of credential IDs to delete from the profile")
    cookies_to_add: Optional[List[Cookie]] = Field(default=None, description="List of cookies to add to the profile")
    cookies_to_delete: Optional[List[StrictStr]] = Field(default=None, description="List of cookie IDs to delete from the profile")
    tracing_snapshots: Optional[StrictBool] = Field(default=True, description="Whether to enable tracing snapshots for the profile")
    extra_stealth: Optional[StrictBool] = Field(default=False, description="Whether to enable extra stealth for the profile")
    __properties: ClassVar[List[str]] = ["name", "description", "proxy_cc", "proxy_type", "captcha_solver_active", "sticky_ip", "credentials_to_add", "credentials_to_delete", "cookies_to_add", "cookies_to_delete", "tracing_snapshots", "extra_stealth"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UpdateAgentProfileRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in credentials_to_add (list)
        _items = []
        if self.credentials_to_add:
            for _item_credentials_to_add in self.credentials_to_add:
                if _item_credentials_to_add:
                    _items.append(_item_credentials_to_add.to_dict())
            _dict['credentials_to_add'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in cookies_to_add (list)
        _items = []
        if self.cookies_to_add:
            for _item_cookies_to_add in self.cookies_to_add:
                if _item_cookies_to_add:
                    _items.append(_item_cookies_to_add.to_dict())
            _dict['cookies_to_add'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UpdateAgentProfileRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "description": obj.get("description"),
            "proxy_cc": obj.get("proxy_cc"),
            "proxy_type": obj.get("proxy_type"),
            "captcha_solver_active": obj.get("captcha_solver_active"),
            "sticky_ip": obj.get("sticky_ip"),
            "credentials_to_add": [Credential.from_dict(_item) for _item in obj["credentials_to_add"]] if obj.get("credentials_to_add") is not None else None,
            "credentials_to_delete": obj.get("credentials_to_delete"),
            "cookies_to_add": [Cookie.from_dict(_item) for _item in obj["cookies_to_add"]] if obj.get("cookies_to_add") is not None else None,
            "cookies_to_delete": obj.get("cookies_to_delete"),
            "tracing_snapshots": obj.get("tracing_snapshots") if obj.get("tracing_snapshots") is not None else True,
            "extra_stealth": obj.get("extra_stealth") if obj.get("extra_stealth") is not None else False
        })
        return _obj





# coding: utf-8

"""
    Agent Service

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AgentsExecutionStatus(str, Enum):
    """
    AgentsExecutionStatus
    """

    """
    allowed enum values
    """
    STARTING = 'starting'
    RUNNING = 'running'
    PAUSED = 'paused'
    AWAITING_CONFIRMATION = 'awaiting_confirmation'
    COMPLETED = 'completed'
    CANCELLED = 'cancelled'
    FAILED = 'failed'
    PAUSED_BY_AGENT = 'paused_by_agent'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AgentsExecutionStatus from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Asteroid Agents API

    Version 1 of the Asteroid Agents API

    The version of the OpenAPI document: v1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from asteroid_odyssey.agents_v1_gen.models.cookie import Cookie
from asteroid_odyssey.agents_v1_gen.models.country_code import CountryCode
from asteroid_odyssey.agents_v1_gen.models.credential import Credential
from asteroid_odyssey.agents_v1_gen.models.proxy_type import ProxyType
from typing import Optional, Set
from typing_extensions import Self

class CreateAgentProfileRequest(BaseModel):
    """
    CreateAgentProfileRequest
    """ # noqa: E501
    name: Annotated[str, Field(min_length=1, strict=True, max_length=255)] = Field(description="Name of the agent profile (must be unique within organization)")
    description: Annotated[str, Field(strict=True, max_length=1000)] = Field(description="Description of the agent profile")
    organization_id: StrictStr = Field(description="The ID of the organization that the agent profile belongs to")
    proxy_cc: CountryCode
    proxy_type: ProxyType
    captcha_solver_active: StrictBool = Field(description="Whether the captcha solver should be active for this profile")
    sticky_ip: StrictBool = Field(description="Whether the same IP address should be used for all executions of this profile")
    credentials: List[Credential] = Field(description="Optional list of credentials to create with the profile")
    cookies: List[Cookie] = Field(description="Optional list of cookies to create with the profile")
    tracing_snapshots: Optional[StrictBool] = Field(default=True, description="Whether to enable tracing snapshots for the profile")
    extra_stealth: Optional[StrictBool] = Field(default=False, description="Whether to enable extra stealth for the profile")
    __properties: ClassVar[List[str]] = ["name", "description", "organization_id", "proxy_cc", "proxy_type", "captcha_solver_active", "sticky_ip", "credentials", "cookies", "tracing_snapshots", "extra_stealth"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateAgentProfileRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in credentials (list)
        _items = []
        if self.credentials:
            for _item_credentials in self.credentials:
                if _item_credentials:
                    _items.append(_item_credentials.to_dict())
            _dict['credentials'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in cookies (list)
        _items = []
        if self.cookies:
            for _item_cookies in self.cookies:
                if _item_cookies:
                    _items.append(_item_cookies.to_dict())
            _dict['cookies'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateAgentProfileRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "description": obj.get("description"),
            "organization_id": obj.get("organization_id"),
            "proxy_cc": obj.get("proxy_cc"),
            "proxy_type": obj.get("proxy_type"),
            "captcha_solver_active": obj.get("captcha_solver_active") if obj.get("captcha_solver_active") is not None else False,
            "sticky_ip": obj.get("sticky_ip") if obj.get("sticky_ip") is not None else False,
            "credentials": [Credential.from_dict(_item) for _item in obj["credentials"]] if obj.get("credentials") is not None else None,
            "cookies": [Cookie.from_dict(_item) for _item in obj["cookies"]] if obj.get("cookies") is not None else None,
            "tracing_snapshots": obj.get("tracing_snapshots") if obj.get("tracing_snapshots") is not None else True,
            "extra_stealth": obj.get("extra_stealth") if obj.get("extra_stealth") is not None else False
        })
        return _obj





# coding: utf-8

# flake8: noqa
"""
    Asteroid Agents API

    Version 1 of the Asteroid Agents API

    The version of the OpenAPI document: v1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from asteroid_odyssey.agents_v1_gen.models.agent_profile import AgentProfile
from asteroid_odyssey.agents_v1_gen.models.browser_session_recording_response import BrowserSessionRecordingResponse
from asteroid_odyssey.agents_v1_gen.models.cookie import Cookie
from asteroid_odyssey.agents_v1_gen.models.country_code import CountryCode
from asteroid_odyssey.agents_v1_gen.models.create_agent_profile_request import CreateAgentProfileRequest
from asteroid_odyssey.agents_v1_gen.models.credential import Credential
from asteroid_odyssey.agents_v1_gen.models.delete_agent_profile200_response import DeleteAgentProfile200Response
from asteroid_odyssey.agents_v1_gen.models.error_response import ErrorResponse
from asteroid_odyssey.agents_v1_gen.models.execution_response import ExecutionResponse
from asteroid_odyssey.agents_v1_gen.models.execution_result import ExecutionResult
from asteroid_odyssey.agents_v1_gen.models.execution_result_response import ExecutionResultResponse
from asteroid_odyssey.agents_v1_gen.models.execution_status_response import ExecutionStatusResponse
from asteroid_odyssey.agents_v1_gen.models.health_check200_response import HealthCheck200Response
from asteroid_odyssey.agents_v1_gen.models.health_check500_response import HealthCheck500Response
from asteroid_odyssey.agents_v1_gen.models.proxy_type import ProxyType
from asteroid_odyssey.agents_v1_gen.models.status import Status
from asteroid_odyssey.agents_v1_gen.models.structured_agent_execution_request import StructuredAgentExecutionRequest
from asteroid_odyssey.agents_v1_gen.models.update_agent_profile_request import UpdateAgentProfileRequest
from asteroid_odyssey.agents_v1_gen.models.upload_execution_files200_response import UploadExecutionFiles200Response



"""
Asteroid Agents Python SDK - High-Level Client Interface

Provides a clean, easy-to-use interface for interacting with the Asteroid Agents API,
similar to the TypeScript SDK.

This module provides a high-level client that wraps the generated OpenAPI client
without modifying any generated files.
"""

import time
import os
import base64
import requests
from pathlib import Path
from typing import Dict, Any, Optional, List, Union, Tuple, NamedTuple
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import padding, rsa
from .agents_v1_gen import (
    Configuration as AgentsV1Configuration,
    ApiClient as AgentsV1ApiClient,
    APIApi as AgentsV1APIApi,
    ExecutionApi as AgentsV1ExecutionApi,
    AgentProfileApi as AgentsV1AgentProfileApi,
    ExecutionStatusResponse,
    ExecutionResult,
    UploadExecutionFiles200Response,
    Status,
    StructuredAgentExecutionRequest,
    CreateAgentProfileRequest,
    UpdateAgentProfileRequest,
    DeleteAgentProfile200Response,
    AgentProfile,
    Credential,
)
from .agents_v1_gen.exceptions import ApiException
from .agents_v2_gen import (
    AgentsAgentBase as Agent,
    AgentList200Response as AgentList200Response,
    Configuration as AgentsV2Configuration,
    ApiClient as AgentsV2ApiClient,
    DefaultApi as AgentsV2ExecutionApi,
    AgentsExecutionActivity as ExecutionActivity,
    AgentsExecutionUserMessagesAddTextBody as ExecutionUserMessagesAddTextBody,
    AgentsFilesFile as File,
)


class AsteroidAPIError(Exception):
    """Base exception for all Asteroid API related errors."""
    pass


class ExecutionError(AsteroidAPIError):
    """Raised when an execution fails or is cancelled."""
    def __init__(self, message: str, execution_result: Optional[ExecutionResult] = None):
        super().__init__(message)
        self.execution_result = execution_result


class TimeoutError(AsteroidAPIError):
    """Raised when an execution times out."""
    def __init__(self, message: str):
        super().__init__(message)


class AgentInteractionResult(NamedTuple):
    """Result returned by wait_for_agent_interaction method."""
    is_terminal: bool  # True if execution reached a terminal state
    status: str  # Current execution status
    agent_message: Optional[str]  # Agent's message if requesting interaction
    execution_result: Optional[ExecutionResult]  # Final result if terminal


def encrypt_with_public_key(plaintext: str, pem_public_key: str) -> str:
    """
    Encrypt plaintext using RSA public key with PKCS1v15 padding.

    Args:
        plaintext: The string to encrypt
        pem_public_key: PEM-formatted RSA public key

    Returns:
        Base64-encoded encrypted string

    Raises:
        ValueError: If encryption fails or key is invalid

    Example:
        encrypted = encrypt_with_public_key("my_password", public_key_pem)
    """
    try:
        # Load the PEM public key (matches node-forge behavior)
        public_key = serialization.load_pem_public_key(pem_public_key.encode('utf-8'))

        if not isinstance(public_key, rsa.RSAPublicKey):
            raise ValueError("Invalid RSA public key")

        # Encrypt using PKCS1v15 padding (matches "RSAES-PKCS1-V1_5" from TypeScript)
        encrypted_bytes = public_key.encrypt(
            plaintext.encode('utf-8'),
            padding.PKCS1v15()
        )

        # Encode as base64 (matches forge.util.encode64)
        return base64.b64encode(encrypted_bytes).decode('utf-8')

    except Exception as e:
        raise ValueError(f"Failed to encrypt: {str(e)}") from e


class AsteroidClient:
    """
    High-level client for the Asteroid Agents API.

    This class provides a convenient interface for executing agents and managing
    their execution lifecycle, similar to the TypeScript SDK.
    """

    def __init__(self, api_key: str, base_url: Optional[str] = None):
        """
        Create an API client with the provided API key.

        Args:
            api_key: Your API key for authentication
            base_url: Optional base URL (defaults to https://odyssey.asteroid.ai/api/v1)

        Example:
            client = AsteroidClient('your-api-key')
        """
        if api_key is None:
            raise TypeError("API key cannot be None")

        # Configure the API client
        config = AgentsV1Configuration(
            host=base_url or "https://odyssey.asteroid.ai/api/v1",
            api_key={'ApiKeyAuth': api_key}
        )

        self.api_client = AgentsV1ApiClient(config)
        self.api_api = AgentsV1APIApi(self.api_client)
        self.execution_api = AgentsV1ExecutionApi(self.api_client)
        self.agent_profile_api = AgentsV1AgentProfileApi(self.api_client)

        self.agents_v2_config = AgentsV2Configuration(
            host=base_url or "https://odyssey.asteroid.ai/agents/v2",
            api_key={'ApiKeyAuth': api_key}
        )
        self.agents_v2_api_client = AgentsV2ApiClient(self.agents_v2_config)
        self.agents_v2_execution_api = AgentsV2ExecutionApi(self.agents_v2_api_client)

    # --- V1 ---

    def execute_agent(self, agent_id: str, execution_data: Dict[str, Any], agent_profile_id: Optional[str] = None) -> str:
        """
        Execute an agent with the provided parameters.

        Args:
            agent_id: The ID of the agent to execute
            execution_data: The execution parameters
            agent_profile_id: Optional ID of the agent profile

        Returns:
            The execution ID

        Raises:
            AsteroidAPIError: If the execution request fails

        Example:
            execution_id = client.execute_agent('my-agent-id', {'input': 'some dynamic value'}, 'agent-profile-id')
        """
        req = StructuredAgentExecutionRequest(dynamic_data=execution_data, agent_profile_id=agent_profile_id)
        try:
            response = self.execution_api.execute_agent_structured(agent_id, req)
            return response.execution_id
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to execute agent: {e}") from e

    def get_execution_status(self, execution_id: str) -> ExecutionStatusResponse:
        """
        Get the current status for an execution.

        Args:
            execution_id: The execution identifier

        Returns:
            The execution status details

        Raises:
            AsteroidAPIError: If the status request fails

        Example:
            status = client.get_execution_status(execution_id)
            print(status.status)
        """
        try:
            return self.execution_api.get_execution_status(execution_id)
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to get execution status: {e}") from e

    def get_execution_result(self, execution_id: str) -> ExecutionResult:
        """
        Get the final result of an execution.

        Args:
            execution_id: The execution identifier

        Returns:
            The execution result object

        Raises:
            AsteroidAPIError: If the result request fails or execution failed

        Example:
            result = client.get_execution_result(execution_id)
            print(result.outcome, result.reasoning)
        """
        try:
            response = self.execution_api.get_execution_result(execution_id)

            if response.error:
                raise AsteroidAPIError(response.error)

            # Handle case where execution_result might be None or have invalid data
            if response.execution_result is None:
                raise AsteroidAPIError("Execution result is not available yet")

            return response.execution_result
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to get execution result: {e}") from e
        except Exception as e:
            # Handle validation errors from ExecutionResult model
            if "must be one of enum values" in str(e):
                raise AsteroidAPIError("Execution result is not available yet - execution may still be running") from e
            raise e

    def wait_for_execution_result(
        self,
        execution_id: str,
        interval: float = 1.0,
        timeout: float = 3600.0
    ) -> ExecutionResult:
        """
        Wait for an execution to reach a terminal state and return the result.

        Continuously polls the execution status until it's either "completed",
        "cancelled", or "failed".

        Args:
            execution_id: The execution identifier
            interval: Polling interval in seconds (default is 1.0)
            timeout: Maximum wait time in seconds (default is 3600 - 1 hour)

        Returns:
            The execution result object

        Raises:
            ValueError: If interval or timeout parameters are invalid
            TimeoutError: If the execution times out
            ExecutionError: If the execution ends as "cancelled" or "failed"

        Example:
            result = client.wait_for_execution_result(execution_id, interval=2.0)
            print(result.outcome, result.reasoning)
        """
        # Validate input parameters
        if interval <= 0:
            raise ValueError("interval must be positive")
        if timeout <= 0:
            raise ValueError("timeout must be positive")
        start_time = time.time()

        while True:
            elapsed_time = time.time() - start_time
            if elapsed_time >= timeout:
                raise TimeoutError(f"Execution {execution_id} timed out after {timeout}s")

            status_response = self.get_execution_status(execution_id)
            current_status = status_response.status

            if current_status == Status.COMPLETED:
                try:
                    return self.get_execution_result(execution_id)
                except Exception as e:
                    if "not available yet" in str(e):
                        # Execution completed but result not ready yet, wait a bit more
                        time.sleep(interval)
                        continue
                    raise e
            elif current_status in [Status.FAILED, Status.CANCELLED]:
                # Get the execution result to provide outcome and reasoning
                try:
                    execution_result = self.get_execution_result(execution_id)
                    reason = f" - {status_response.reason}" if status_response.reason else ""
                    raise ExecutionError(
                        f"Execution {execution_id} ended with status: {current_status.value}{reason}",
                        execution_result
                    )
                except Exception as e:
                    # If we can't get the execution result, fall back to the original behavior
                    reason = f" - {status_response.reason}" if status_response.reason else ""
                    raise ExecutionError(f"Execution {execution_id} ended with status: {current_status.value}{reason}") from e

            # Wait for the specified interval before polling again
            time.sleep(interval)

    def upload_execution_files(
        self,
        execution_id: str,
        files: List[Union[bytes, str, Tuple[str, bytes]]],
        default_filename: str = "file.txt"
    ) -> UploadExecutionFiles200Response:
        """
        Upload files to an execution.

        Args:
            execution_id: The execution identifier
            files: List of files to upload. Each file can be:
                   - bytes: Raw file content (will use default_filename)
                   - str: File path as string (will read file and use filename)
                   - Tuple[str, bytes]: (filename, file_content) tuple
            default_filename: Default filename to use when file is provided as bytes

        Returns:
            The upload response containing message and file IDs

        Raises:
            Exception: If the upload request fails

        Example:
            # Upload with file content (file should be in your current working directory)
            with open('hello.txt', 'r') as f:
                file_content = f.read()

            response = client.upload_execution_files(execution_id, [file_content.encode()])
            print(f"Uploaded files: {response.file_ids}")

            # Upload with filename and content
            files = [('hello.txt', file_content.encode())]
            response = client.upload_execution_files(execution_id, files)

            # Or create content directly
            hello_content = "Hello World!".encode()
            response = client.upload_execution_files(execution_id, [hello_content])
        """
        try:
            # Process files to ensure proper format
            processed_files = []
            for file_item in files:
                if isinstance(file_item, tuple):
                    # Already in (filename, content) format
                    filename, content = file_item
                    if isinstance(content, str):
                        content = content.encode()
                    processed_files.append((filename, content))
                elif isinstance(file_item, str):
                    # Check if string is a file path that exists, otherwise treat as content
                    if os.path.isfile(file_item):
                        # File path - read the file
                        filename = os.path.basename(file_item)
                        with open(file_item, 'rb') as f:
                            content = f.read()
                        processed_files.append((filename, content))
                    else:
                        # String content - encode and use default filename
                        content = file_item.encode()
                        processed_files.append((default_filename, content))
                elif isinstance(file_item, bytes):
                    # Raw bytes - use default filename
                    processed_files.append((default_filename, file_item))
                else:
                    # Other types - convert to string content and encode
                    content = str(file_item).encode()
                    processed_files.append((default_filename, content))

            response = self.execution_api.upload_execution_files(execution_id, files=processed_files)
            return response
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to upload execution files: {e}") from e

    def get_browser_session_recording(self, execution_id: str) -> str:
        """
        Get the browser session recording URL for a completed execution.

        Args:
            execution_id: The execution identifier

        Returns:
            The URL of the browser session recording

        Raises:
            Exception: If the recording request fails

        Example:
            recording_url = client.get_browser_session_recording(execution_id)
            print(f"Recording available at: {recording_url}")
        """
        try:
            response = self.execution_api.get_browser_session_recording(execution_id)
            return response.recording_url
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to get browser session recording: {e}") from e

    def get_agent_profiles(self, organization_id: str) -> List[AgentProfile]:
        """
        Get a list of agent profiles for a specific organization.

        Args:
            organization_id: The organization identifier (required)
        Returns:
            A list of agent profiles
        Raises:
            Exception: If the agent profiles request fails
        Example:
            profiles = client.get_agent_profiles("org-123")
        """
        try:
            response = self.agent_profile_api.get_agent_profiles(organization_id=organization_id)
            return response  # response is already a List[AgentProfile]
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to get agent profiles: {e}") from e
    def get_agent_profile(self, profile_id: str) -> AgentProfile:
        """
        Get an agent profile by ID.
        Args:
            profile_id: The ID of the agent profile
        Returns:
            The agent profile
        Raises:
            Exception: If the agent profile request fails
        Example:
            profile = client.get_agent_profile("profile_id")
        """
        try:
            response = self.agent_profile_api.get_agent_profile(profile_id)
            return response
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to get agent profile: {e}") from e

    def create_agent_profile(self, request: CreateAgentProfileRequest) -> AgentProfile:
        """
        Create an agent profile with automatic credential encryption.

        Args:
            request: The request object
        Returns:
            The agent profile
        Raises:
            Exception: If the agent profile creation fails
        Example:
            request = CreateAgentProfileRequest(
                name="My Agent Profile",
                description="This is my agent profile",
                organization_id="org-123",
                proxy_cc=CountryCode.US,
                proxy_type=ProxyType.RESIDENTIAL,
                captcha_solver_active=True,
                sticky_ip=True,
                credentials=[Credential(name="user", data="password")]
            )
            profile = client.create_agent_profile(request)
        """
        try:
            # Create a copy to avoid modifying the original request
            processed_request = request

            # If credentials are provided, encrypt them before sending
            if request.credentials and len(request.credentials) > 0:
                # Get the public key for encryption
                public_key = self.get_credentials_public_key()

                # Encrypt each credential's data field
                encrypted_credentials = []
                for credential in request.credentials:
                    encrypted_credential = Credential(
                        name=credential.name,
                        data=encrypt_with_public_key(credential.data, public_key),
                        id=credential.id,
                        created_at=credential.created_at
                    )
                    encrypted_credentials.append(encrypted_credential)

                # Create new request with encrypted credentials
                processed_request = CreateAgentProfileRequest(
                    name=request.name,
                    description=request.description,
                    organization_id=request.organization_id,
                    proxy_cc=request.proxy_cc,
                    proxy_type=request.proxy_type,
                    captcha_solver_active=request.captcha_solver_active,
                    sticky_ip=request.sticky_ip,
                    credentials=encrypted_credentials
                )

            response = self.agent_profile_api.create_agent_profile(processed_request)
            return response
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to create agent profile: {e}") from e
    def update_agent_profile(self, profile_id: str, request: UpdateAgentProfileRequest) -> AgentProfile:
        """
        Update an agent profile with automatic credential encryption.

        Args:
            profile_id: The ID of the agent profile
            request: The request object
        Returns:
            The agent profile
        Raises:
            Exception: If the agent profile update fails
        Example:
            request = UpdateAgentProfileRequest(
                name="My Agent Profile",
                description="This is my agent profile",
                credentials_to_add=[Credential(name="api_key", data="secret")]
            )
            profile = client.update_agent_profile("profile_id", request)
        """
        try:
            # Create a copy to avoid modifying the original request
            processed_request = request

            # If credentials_to_add are provided, encrypt them before sending
            if request.credentials_to_add and len(request.credentials_to_add) > 0:
                # Get the public key for encryption
                public_key = self.get_credentials_public_key()

                # Encrypt the data field of each credential to add
                encrypted_credentials_to_add = []
                for credential in request.credentials_to_add:
                    encrypted_credential = Credential(
                        name=credential.name,
                        data=encrypt_with_public_key(credential.data, public_key),
                        id=credential.id,
                        created_at=credential.created_at
                    )
                    encrypted_credentials_to_add.append(encrypted_credential)

                # Create new request with encrypted credentials
                processed_request = UpdateAgentProfileRequest(
                    name=request.name,
                    description=request.description,
                    proxy_cc=request.proxy_cc,
                    proxy_type=request.proxy_type,
                    captcha_solver_active=request.captcha_solver_active,
                    sticky_ip=request.sticky_ip,
                    credentials_to_add=encrypted_credentials_to_add,
                    credentials_to_delete=request.credentials_to_delete
                )

            response = self.agent_profile_api.update_agent_profile(profile_id, processed_request)
            return response
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to update agent profile: {e}") from e
    def delete_agent_profile(self, profile_id: str) -> DeleteAgentProfile200Response:
        """
        Delete an agent profile.
        Args:
            profile_id: The ID of the agent profile
        Returns:
            Confirmation message from the server
        Raises:
            Exception: If the agent profile deletion fails
        Example:
            response = client.delete_agent_profile("profile_id")
        """
        try:
            response = self.agent_profile_api.delete_agent_profile(profile_id)
            return response
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to delete agent profile: {e}") from e

    def get_credentials_public_key(self) -> str:
        """
        Get the public key for encrypting credentials.

        Returns:
            PEM-formatted RSA public key string

        Raises:
            Exception: If the public key request fails

        Example:
            public_key = client.get_credentials_public_key()
        """
        try:
            response = self.agent_profile_api.get_credentials_public_key()
            return response
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to get credentials public key: {e}") from e

    def wait_for_agent_interaction(
        self,
        execution_id: str,
        poll_interval: float = 2.0,
        timeout: float = 3600.0
    ) -> AgentInteractionResult:
        """
        Wait for an agent interaction request or terminal state.

        This method polls an existing execution until it either:
        1. Requests human input (paused_by_agent state)
        2. Reaches a terminal state (completed, failed, cancelled)
        3. Times out

        Unlike interactive_agent, this method doesn't start an execution or handle
        the response automatically - it just waits and reports what happened.

        Args:
            execution_id: The execution identifier for an already started execution
            poll_interval: How often to check for updates in seconds (default: 2.0)
            timeout: Maximum wait time in seconds (default: 3600 - 1 hour)

        Returns:
            AgentInteractionResult containing:
            - is_terminal: True if execution finished (completed/failed/cancelled)
            - status: Current execution status string
            - agent_message: Agent's message if requesting interaction (None if terminal)
            - execution_result: Final result if terminal state (None if requesting interaction)

        Raises:
            ValueError: If interval or timeout parameters are invalid
            TimeoutError: If the execution times out
            AsteroidAPIError: If API calls fail

        Example:
            # Start an execution first
            execution_id = client.execute_agent('agent-id', {'input': 'test'})

            # Wait for interaction or completion
            result = client.wait_for_agent_interaction(execution_id)

            if result.is_terminal:
                print(f"Execution finished with status: {result.status}")
                if result.execution_result:
                    print(f"Result: {result.execution_result.outcome}")
            else:
                print(f"Agent requesting input: {result.agent_message}")
                # Send response
                client.add_message_to_execution(execution_id, "user response")
                # Wait again
                result = client.wait_for_agent_interaction(execution_id)
        """
        # Validate parameters
        if poll_interval <= 0:
            raise ValueError("poll_interval must be positive")
        if timeout <= 0:
            raise ValueError("timeout must be positive")

        start_time = time.time()

        while True:
            elapsed_time = time.time() - start_time
            if elapsed_time >= timeout:
                raise TimeoutError(f"Wait for interaction on execution {execution_id} timed out after {timeout}s")

            # Get current status
            status_response = self.get_execution_status(execution_id)
            current_status = status_response.status
            status_str = current_status.value.lower()

            # Handle terminal states
            if current_status == Status.COMPLETED:
                try:
                    execution_result = self.get_execution_result(execution_id)
                    return AgentInteractionResult(
                        is_terminal=True,
                        status=status_str,
                        agent_message=None,
                        execution_result=execution_result
                    )
                except AsteroidAPIError as e:
                    if "not available yet" in str(e):
                        time.sleep(poll_interval)
                        continue
                    raise e

            elif current_status in [Status.FAILED, Status.CANCELLED]:
                try:
                    execution_result = self.get_execution_result(execution_id)
                    return AgentInteractionResult(
                        is_terminal=True,
                        status=status_str,
                        agent_message=None,
                        execution_result=execution_result
                    )
                except AsteroidAPIError as e:
                    # If we can't get the execution result, still return terminal state
                    return AgentInteractionResult(
                        is_terminal=True,
                        status=status_str,
                        agent_message=None,
                        execution_result=None
                    )

            # Handle agent interaction request
            elif current_status == Status.PAUSED_BY_AGENT:
                # Get the agent's message/request
                agent_message = self._extract_agent_request_message(execution_id)
                return AgentInteractionResult(
                    is_terminal=False,
                    status=status_str,
                    agent_message=agent_message,
                    execution_result=None
                )

            # Wait before next poll for non-terminal, non-interaction states
            time.sleep(poll_interval)

    def _extract_agent_request_message(self, execution_id: str) -> str:
        """
        Extract the agent's request message from recent activities.

        Args:
            execution_id: The execution identifier

        Returns:
            The agent's message or a default message if not found
        """
        try:
            activities = self.get_last_n_execution_activities(execution_id, 20)

            # Filter for human input requests
            human_input_requests = [
                activity for activity in activities
                if (hasattr(activity, 'payload') and
                    activity.payload and
                    getattr(activity.payload, 'activityType', None) == 'action_started')
            ]

            if human_input_requests:
                human_input_request = human_input_requests[0]

                # Extract message from payload data with robust error handling
                try:
                    payload = human_input_request.payload
                    if hasattr(payload, 'data') and payload.data:
                        payload_data = payload.data
                        if hasattr(payload_data, 'message') and payload_data.message:
                            return str(payload_data.message)
                    return 'Agent is requesting input'
                except (AttributeError, TypeError) as e:
                    return 'Agent is requesting input (extraction failed)'

            return 'Agent is requesting input'

        except AsteroidAPIError as e:
            return 'Agent is requesting input (API error)'
        except Exception as e:
            return 'Agent is requesting input (extraction failed)'

    def __enter__(self):
        """Context manager entry."""
        return self

    def __exit__(self, exc_type, exc_value, tb):
        """Context manager exit: clean up API client connection pool."""
        try:
            # Try to grab the pool_manager; if any attr is missing, skip
            try:
                pool_manager = self.api_client.rest_client.pool_manager
            except AttributeError:
                pool_manager = None

            if pool_manager:
                pool_manager.clear()
        except Exception as e:
            pass
        return False

    # --- V2 ---

    def get_agents(self, org_id: str, page: int = 1, page_size: int = 100) -> List[Agent]:
        """
        Get a paginated list of agents for an organization.
        Args:
            org_id: The organization identifier
            page: The page number
            page_size: The page size
        Returns:
            A list of agents
        Raises:
            Exception: If the agents request fails
        Example:
            agents = client.get_agents("org_id", page=1, page_size=100)
            for agent in agents:
                print(f"Agent: {agent.name}")
        """
        response = self.agents_v2_execution_api.agent_list(organization_id=org_id, page=page, page_size=page_size)
        return response.items

    def get_last_n_execution_activities(self, execution_id: str, n: int) -> List[ExecutionActivity]:
        """
        Get the last N execution activities for a given execution ID, sorted by their timestamp in descending order.
        Args:
            execution_id: The execution identifier
            n: The number of activities to return
        Returns:
            A list of execution activities
        Raises:
            Exception: If the execution activities request fails
        """
        return self.agents_v2_execution_api.execution_activities_get(execution_id, order="desc", limit=n)
    def add_message_to_execution(self, execution_id: str, message: str) -> None:
        """
        Add a message to an execution.
        Args:
            execution_id: The execution identifier
            message: The message to add
        Returns:
            None
        Raises:
            Exception: If the message addition fails
        Example:
            add_message_to_execution(client, "execution_id", "Hello, world!")
        """
        message_body = ExecutionUserMessagesAddTextBody(message=message)
        return self.agents_v2_execution_api.execution_user_messages_add(execution_id, message_body)

    def get_execution_files(self, execution_id: str) -> List[File]:
        """
        Get a list of files associated with an execution.
        Args:
            execution_id: The execution identifier
        Returns:
            A list of files associated with the execution
        Raises:
            Exception: If the files request fails
        Example:
            files = client.get_execution_files("execution_id")
            for file in files:
                print(f"File: {file.file_name}, Size: {file.file_size}")
        """
        try:
            return self.agents_v2_execution_api.execution_context_files_get(execution_id)
        except ApiException as e:
            raise AsteroidAPIError(f"Failed to get execution files: {e}") from e

    def download_execution_file(self, file: File, download_path: Union[str, Path],
                              create_dirs: bool = True, timeout: int = 30) -> str:
        """
        Download a file from an execution using its signed URL.

        Args:
            file: The File object containing the signed URL and metadata
            download_path: Path where the file should be saved. Can be a directory or full file path
            create_dirs: Whether to create parent directories if they don't exist (default: True)
            timeout: Request timeout in seconds (default: 30)

        Returns:
            The full path where the file was saved

        Raises:
            AsteroidAPIError: If the download fails
            FileNotFoundError: If the parent directory doesn't exist and create_dirs is False

        Example:
            files = client.get_execution_files("execution_id")
            for file in files:
                # Download to specific directory
                saved_path = client.download_execution_file(file, "/path/to/downloads/")
                print(f"Downloaded {file.file_name} to {saved_path}")

                # Download with specific filename
                saved_path = client.download_execution_file(file, "/path/to/downloads/my_file.txt")
                print(f"Downloaded to {saved_path}")
        """
        final_path = None
        try:
            # Convert to Path object for easier manipulation
            download_path = Path(download_path)

            # Determine the final file path
            if download_path.is_dir() or str(download_path).endswith('/'):
                # If download_path is a directory, use the original filename
                final_path = download_path / file.file_name
            else:
                # If download_path includes a filename, use it as-is
                final_path = download_path

            # Create parent directories if needed
            if create_dirs:
                final_path.parent.mkdir(parents=True, exist_ok=True)
            elif not final_path.parent.exists():
                raise FileNotFoundError(f"Parent directory does not exist: {final_path.parent}")

            # Download the file using the signed URL
            response = requests.get(file.signed_url, timeout=timeout, stream=True)
            response.raise_for_status()

            # Verify content length if available
            expected_size = file.file_size
            content_length = response.headers.get('content-length')
            if content_length and int(content_length) != expected_size:
                raise AsteroidAPIError(
                    f"Content length mismatch: expected {expected_size}, got {content_length}"
                )

            # Write the file in chunks to handle large files efficiently
            chunk_size = 8192
            total_size = 0

            with open(final_path, 'wb') as f:
                for chunk in response.iter_content(chunk_size=chunk_size):
                    if chunk:  # Filter out keep-alive chunks
                        f.write(chunk)
                        total_size += len(chunk)

            # Final verification of the downloaded file size
            if total_size != expected_size:
                raise AsteroidAPIError(
                    f"Downloaded file size mismatch: expected {expected_size}, got {total_size}"
                )

            return str(final_path)

        except requests.exceptions.RequestException as e:
            # Clean up partial file on network error
            if final_path and final_path.exists():
                final_path.unlink(missing_ok=True)
            raise AsteroidAPIError(f"Failed to download file {file.file_name}: {e}") from e
        except OSError as e:
            # Clean up partial file on I/O error
            if final_path and final_path.exists():
                final_path.unlink(missing_ok=True)
            raise AsteroidAPIError(f"Failed to save file {file.file_name}: {e}") from e
        except AsteroidAPIError:
            # Clean up partial file on size mismatch or other API errors
            if final_path and final_path.exists():
                final_path.unlink(missing_ok=True)
            raise
        except Exception as e:
            # Clean up partial file on unexpected error
            if final_path and final_path.exists():
                final_path.unlink(missing_ok=True)
            raise AsteroidAPIError(f"Unexpected error downloading file {file.file_name}: {e}") from e

# Convenience functions that mirror the TypeScript SDK pattern
def create_client(api_key: str, base_url: Optional[str] = None) -> AsteroidClient:
    """
    Create an API client with a provided API key.

    This is a convenience function that creates an AsteroidClient instance.

    Args:
        api_key: Your API key
        base_url: Optional base URL

    Returns:
        A configured AsteroidClient instance

    Example:
        client = create_client('your-api-key')
    """
    return AsteroidClient(api_key, base_url)

# --- V1 ---

def execute_agent(client: AsteroidClient, agent_id: str, execution_data: Dict[str, Any], agent_profile_id: Optional[str] = None) -> str:
    """
    Execute an agent with the provided parameters.

    Args:
        client: The AsteroidClient instance
        agent_id: The ID of the agent to execute
        execution_data: The execution parameters
        agent_profile_id: Optional ID of the agent profile

    Returns:
        The execution ID

    Example:
        execution_id = execute_agent(client, 'my-agent-id', {'input': 'some dynamic value'}, 'agent-profile-id')
    """
    return client.execute_agent(agent_id, execution_data, agent_profile_id)



def get_execution_status(client: AsteroidClient, execution_id: str) -> ExecutionStatusResponse:
    """
    Get the current status for an execution.

    Args:
        client: The AsteroidClient instance
        execution_id: The execution identifier

    Returns:
        The execution status details

    Example:
        status = get_execution_status(client, execution_id)
        print(status.status)
    """
    return client.get_execution_status(execution_id)


def get_execution_result(client: AsteroidClient, execution_id: str) -> ExecutionResult:
    """
    Get the final result of an execution.

    Args:
        client: The AsteroidClient instance
        execution_id: The execution identifier

    Returns:
        The execution result object

    Raises:
        Exception: If the result is not available yet or execution failed

    Example:
        result = get_execution_result(client, execution_id)
        print(result.outcome, result.reasoning)
    """
    return client.get_execution_result(execution_id)


def wait_for_execution_result(
    client: AsteroidClient,
    execution_id: str,
    interval: float = 1.0,
    timeout: float = 3600.0
) -> ExecutionResult:
    """
    Wait for an execution to reach a terminal state and return the result.

    Args:
        client: The AsteroidClient instance
        execution_id: The execution identifier
        interval: Polling interval in seconds (default is 1.0)
        timeout: Maximum wait time in seconds (default is 3600 - 1 hour)

    Returns:
        The execution result object

    Raises:
        TimeoutError: If the execution times out
        ExecutionError: If the execution ends as "cancelled" or "failed"

    Example:
        result = wait_for_execution_result(client, execution_id, interval=2.0)
        print(result.outcome, result.reasoning)
    """
    return client.wait_for_execution_result(execution_id, interval, timeout)


def upload_execution_files(
    client: AsteroidClient,
    execution_id: str,
    files: List[Union[bytes, str, Tuple[str, bytes]]],
    default_filename: str = "file.txt"
) -> UploadExecutionFiles200Response:
    """
    Upload files to an execution.

    Args:
        client: The AsteroidClient instance
        execution_id: The execution identifier
        files: List of files to upload
        default_filename: Default filename to use when file is provided as bytes

    Returns:
        The upload response containing message and file IDs

    Example:
        # Create a simple text file with "Hello World!" content
        hello_content = "Hello World!".encode()
        response = upload_execution_files(client, execution_id, [hello_content])
        print(f"Uploaded files: {response.file_ids}")

        # Or specify filename with content
        files = [('hello.txt', "Hello World!".encode())]
        response = upload_execution_files(client, execution_id, files)
    """
    return client.upload_execution_files(execution_id, files, default_filename)


def get_browser_session_recording(client: AsteroidClient, execution_id: str) -> str:
    """
    Get the browser session recording URL for a completed execution.

    Args:
        client: The AsteroidClient instance
        execution_id: The execution identifier

    Returns:
        The URL of the browser session recording

    Example:
        recording_url = get_browser_session_recording(client, execution_id)
        print(f"Recording available at: {recording_url}")
    """
    return client.get_browser_session_recording(execution_id)

def get_agent_profiles(client: AsteroidClient, organization_id: Optional[str] = None) -> List[AgentProfile]:
    """
    Get a list of agent profiles.
    Args:
        client: The AsteroidClient instance
        organization_id: The organization identifier (optional) Returns all agent profiles if no organization_id is provided.
    Returns:
        A list of agent profiles
    Raises:
        Exception: If the agent profiles request fails
    Example:
        profiles = get_agent_profiles(client, "org-123")
    """
    return client.get_agent_profiles(organization_id)
def get_agent_profile(client: AsteroidClient, profile_id: str) -> AgentProfile:
    """
    Get an agent profile by ID.
    Args:
        client: The AsteroidClient instance
        profile_id: The ID of the agent profile
    Returns:
        The agent profile
    Raises:
        Exception: If the agent profile request fails
    Example:
        profile = get_agent_profile(client, "profile_id")
    """
    return client.get_agent_profile(profile_id)
def create_agent_profile(client: AsteroidClient, request: CreateAgentProfileRequest) -> AgentProfile:
    """
    Create an agent profile.
    Args:
        client: The AsteroidClient instance
        request: The request object
    Returns:
        The agent profile
    Raises:
        Exception: If the agent profile creation fails
    Example:
        request = CreateAgentProfileRequest(
            name="My Agent Profile",
            description="This is my agent profile",
            organization_id="org-123",
            proxy_cc=CountryCode.US,
            proxy_type=ProxyType.RESIDENTIAL,
            captcha_solver_active=True,
            sticky_ip=True,
            credentials=[Credential(name="user", data="password")]
        )
        profile = create_agent_profile(client, request)
    """
    return client.create_agent_profile(request)
def update_agent_profile(client: AsteroidClient, profile_id: str, request: UpdateAgentProfileRequest) -> AgentProfile:
    """
    Update an agent profile with the provided request.
    Args:
        client: The AsteroidClient instance
        profile_id: The ID of the agent profile
        request: The request object
    Returns:
        The agent profile
    Raises:
        Exception: If the agent profile update fails
    Example:
        request = UpdateAgentProfileRequest(
            name="My Agent Profile",
            description="This is my agent profile",
            organization_id="org-123",
        )
        profile = update_agent_profile(client, "profile_id", request)
    """
    return client.update_agent_profile(profile_id, request)
def delete_agent_profile(client: AsteroidClient, profile_id: str) -> DeleteAgentProfile200Response:
    """
    Delete an agent profile.
    Args:
        client: The AsteroidClient instance
        profile_id: The ID of the agent profile
    Returns:
        The agent profile
    Raises:
        Exception: If the agent profile deletion fails
    Example:
        profile_deleted =delete_agent_profile(client, "profile_id")
    """
    return client.delete_agent_profile(profile_id)

def get_credentials_public_key(client: AsteroidClient) -> str:
    """
    Get the public key for encrypting credentials.

    Args:
        client: The AsteroidClient instance

    Returns:
        PEM-formatted RSA public key string

    Example:
        public_key = get_credentials_public_key(client)
    """
    return client.get_credentials_public_key()

# --- V2 ---

def get_agents(client: AsteroidClient, org_id: str, page: int = 1, page_size: int = 100) -> List[Agent]:
    """
    Get a paginated list of agents for an organization.
    Args:
        client: The AsteroidClient instance
        org_id: The organization identifier
        page: The page number
        page_size: The page size
    Returns:
        A list of agents
    Raises:
        Exception: If the agents request fails
    Example:
        agents = get_agents(client, "org_id", page=1, page_size=100)
        for agent in agents:
            print(f"Agent: {agent.name}")
    """
    response = client.get_agents(org_id, page, page_size)
    return response.items
def get_last_n_execution_activities(client: AsteroidClient, execution_id: str, n: int) -> List[ExecutionActivity]:
    """
    Get the last N execution activities for a given execution ID, sorted by their timestamp in descending order.
    Args:
        client: The AsteroidClient instance
        execution_id: The execution identifier
        n: The number of activities to return
    Returns:
        A list of execution activities
    Raises:
        Exception: If the execution activities request fails
    Example:
        activities = get_last_n_execution_activities(client, "execution_id", 10)
    """
    return client.get_last_n_execution_activities(execution_id, n)

def add_message_to_execution(client: AsteroidClient, execution_id: str, message: str) -> None:
    """
    Add a message to an execution.
    Args:
        client: The AsteroidClient instance
        execution_id: The execution identifier
        message: The message to add
    Returns:
        None
    Raises:
        Exception: If the message addition fails
    Example:
        add_message_to_execution(client, "execution_id", "Hello, world!")
    """
    return client.add_message_to_execution(execution_id, message)

def get_execution_files(client: AsteroidClient, execution_id: str) -> List[File]:
    """
    Get a list of files associated with an execution.
    Args:
        client: The AsteroidClient instance
        execution_id: The execution identifier
    Returns:
        A list of files associated with the execution
    Raises:
        Exception: If the files request fails
    Example:
        files = get_execution_files(client, "execution_id")
        for file in files:
            print(f"File: {file.file_name}, Size: {file.file_size}")
    """
    return client.get_execution_files(execution_id)

def download_execution_file(client: AsteroidClient, file: File, download_path: Union[str, Path],
                          create_dirs: bool = True, timeout: int = 30) -> str:
    """
    Download a file from an execution using its signed URL.

    Args:
        client: The AsteroidClient instance
        file: The File object containing the signed URL and metadata
        download_path: Path where the file should be saved. Can be a directory or full file path
        create_dirs: Whether to create parent directories if they don't exist (default: True)
        timeout: Request timeout in seconds (default: 30)

    Returns:
        The full path where the file was saved

    Raises:
        AsteroidAPIError: If the download fails
        FileNotFoundError: If the parent directory doesn't exist and create_dirs is False

    Example:
        files = get_execution_files(client, "execution_id")
        for file in files:
            # Download to specific directory
            saved_path = download_execution_file(client, file, "/path/to/downloads/")
            print(f"Downloaded {file.file_name} to {saved_path}")

            # Download with specific filename
            saved_path = download_execution_file(client, file, "/path/to/downloads/my_file.txt")
            print(f"Downloaded to {saved_path}")
    """
    return client.download_execution_file(file, download_path, create_dirs, timeout)



def wait_for_agent_interaction(
    client: AsteroidClient,
    execution_id: str,
    poll_interval: float = 2.0,
    timeout: float = 3600.0
) -> AgentInteractionResult:
    """
    Wait for an agent interaction request or terminal state.

    This convenience function provides the same functionality as the AsteroidClient.wait_for_agent_interaction method.

    Args:
        client: The AsteroidClient instance
        execution_id: The execution identifier for an already started execution
        poll_interval: How often to check for updates in seconds (default: 2.0)
        timeout: Maximum wait time in seconds (default: 3600 - 1 hour)

    Returns:
        AgentInteractionResult containing:
        - is_terminal: True if execution finished (completed/failed/cancelled)
        - status: Current execution status string
        - agent_message: Agent's message if requesting interaction (None if terminal)
        - execution_result: Final result if terminal state (None if requesting interaction)

    Raises:
        ValueError: If interval or timeout parameters are invalid
        TimeoutError: If the execution times out
        AsteroidAPIError: If API calls fail

    Example:
        # Start an execution first
        execution_id = execute_agent(client, 'agent-id', {'input': 'test'})

        # Wait for interaction or completion
        result = wait_for_agent_interaction(client, execution_id)

        if result.is_terminal:
            print(f"Execution finished with status: {result.status}")
            if result.execution_result:
                print(f"Result: {result.execution_result.outcome}")
        else:
            print(f"Agent requesting input: {result.agent_message}")
            # Send response
            add_message_to_execution(client, execution_id, "user response")
            # Wait again
            result = wait_for_agent_interaction(client, execution_id)
    """
    return client.wait_for_agent_interaction(
        execution_id=execution_id,
        poll_interval=poll_interval,
        timeout=timeout
    )

# Re-export common types for convenience
__all__ = [
    'AsteroidClient',
    'create_client',
    'execute_agent',
    'get_execution_status',
    'get_execution_result',
    'wait_for_execution_result',
    'upload_execution_files',
    'get_browser_session_recording',
    'get_agent_profiles',
    'get_agent_profile',
    'create_agent_profile',
    'update_agent_profile',
    'delete_agent_profile',
    'get_agents',
    'get_last_n_execution_activities',
    'add_message_to_execution',
    'get_execution_files',
    'download_execution_file',
    'wait_for_agent_interaction',
    'get_credentials_public_key',
    'AsteroidAPIError',
    'ExecutionError',
    'TimeoutError',
    'AgentInteractionResult'
]



# coding: utf-8

"""
    Agent Service

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_action_completed import ActivityPayloadUnionActionCompleted
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_action_failed import ActivityPayloadUnionActionFailed
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_action_started import ActivityPayloadUnionActionStarted
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_file_added import ActivityPayloadUnionFileAdded
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_generic import ActivityPayloadUnionGeneric
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_graph_updated import ActivityPayloadUnionGraphUpdated
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_status_changed import ActivityPayloadUnionStatusChanged
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_step_completed import ActivityPayloadUnionStepCompleted
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_step_started import ActivityPayloadUnionStepStarted
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_terminal import ActivityPayloadUnionTerminal
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_transitioned_node import ActivityPayloadUnionTransitionedNode
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_user_message_received import ActivityPayloadUnionUserMessageReceived
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

AGENTSEXECUTIONACTIVITYPAYLOADUNION_ONE_OF_SCHEMAS = ["ActivityPayloadUnionActionCompleted", "ActivityPayloadUnionActionFailed", "ActivityPayloadUnionActionStarted", "ActivityPayloadUnionFileAdded", "ActivityPayloadUnionGeneric", "ActivityPayloadUnionGraphUpdated", "ActivityPayloadUnionStatusChanged", "ActivityPayloadUnionStepCompleted", "ActivityPayloadUnionStepStarted", "ActivityPayloadUnionTerminal", "ActivityPayloadUnionTransitionedNode", "ActivityPayloadUnionUserMessageReceived"]

class AgentsExecutionActivityPayloadUnion(BaseModel):
    """
    AgentsExecutionActivityPayloadUnion
    """
    # data type: ActivityPayloadUnionTerminal
    oneof_schema_1_validator: Optional[ActivityPayloadUnionTerminal] = None
    # data type: ActivityPayloadUnionGeneric
    oneof_schema_2_validator: Optional[ActivityPayloadUnionGeneric] = None
    # data type: ActivityPayloadUnionStepStarted
    oneof_schema_3_validator: Optional[ActivityPayloadUnionStepStarted] = None
    # data type: ActivityPayloadUnionStepCompleted
    oneof_schema_4_validator: Optional[ActivityPayloadUnionStepCompleted] = None
    # data type: ActivityPayloadUnionTransitionedNode
    oneof_schema_5_validator: Optional[ActivityPayloadUnionTransitionedNode] = None
    # data type: ActivityPayloadUnionStatusChanged
    oneof_schema_6_validator: Optional[ActivityPayloadUnionStatusChanged] = None
    # data type: ActivityPayloadUnionActionStarted
    oneof_schema_7_validator: Optional[ActivityPayloadUnionActionStarted] = None
    # data type: ActivityPayloadUnionActionCompleted
    oneof_schema_8_validator: Optional[ActivityPayloadUnionActionCompleted] = None
    # data type: ActivityPayloadUnionActionFailed
    oneof_schema_9_validator: Optional[ActivityPayloadUnionActionFailed] = None
    # data type: ActivityPayloadUnionUserMessageReceived
    oneof_schema_10_validator: Optional[ActivityPayloadUnionUserMessageReceived] = None
    # data type: ActivityPayloadUnionFileAdded
    oneof_schema_11_validator: Optional[ActivityPayloadUnionFileAdded] = None
    # data type: ActivityPayloadUnionGraphUpdated
    oneof_schema_12_validator: Optional[ActivityPayloadUnionGraphUpdated] = None
    actual_instance: Optional[Union[ActivityPayloadUnionActionCompleted, ActivityPayloadUnionActionFailed, ActivityPayloadUnionActionStarted, ActivityPayloadUnionFileAdded, ActivityPayloadUnionGeneric, ActivityPayloadUnionGraphUpdated, ActivityPayloadUnionStatusChanged, ActivityPayloadUnionStepCompleted, ActivityPayloadUnionStepStarted, ActivityPayloadUnionTerminal, ActivityPayloadUnionTransitionedNode, ActivityPayloadUnionUserMessageReceived]] = None
    one_of_schemas: Set[str] = { "ActivityPayloadUnionActionCompleted", "ActivityPayloadUnionActionFailed", "ActivityPayloadUnionActionStarted", "ActivityPayloadUnionFileAdded", "ActivityPayloadUnionGeneric", "ActivityPayloadUnionGraphUpdated", "ActivityPayloadUnionStatusChanged", "ActivityPayloadUnionStepCompleted", "ActivityPayloadUnionStepStarted", "ActivityPayloadUnionTerminal", "ActivityPayloadUnionTransitionedNode", "ActivityPayloadUnionUserMessageReceived" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = AgentsExecutionActivityPayloadUnion.model_construct()
        error_messages = []
        match = 0
        # validate data type: ActivityPayloadUnionTerminal
        if not isinstance(v, ActivityPayloadUnionTerminal):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionTerminal`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionGeneric
        if not isinstance(v, ActivityPayloadUnionGeneric):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionGeneric`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionStepStarted
        if not isinstance(v, ActivityPayloadUnionStepStarted):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionStepStarted`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionStepCompleted
        if not isinstance(v, ActivityPayloadUnionStepCompleted):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionStepCompleted`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionTransitionedNode
        if not isinstance(v, ActivityPayloadUnionTransitionedNode):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionTransitionedNode`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionStatusChanged
        if not isinstance(v, ActivityPayloadUnionStatusChanged):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionStatusChanged`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionActionStarted
        if not isinstance(v, ActivityPayloadUnionActionStarted):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionActionStarted`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionActionCompleted
        if not isinstance(v, ActivityPayloadUnionActionCompleted):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionActionCompleted`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionActionFailed
        if not isinstance(v, ActivityPayloadUnionActionFailed):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionActionFailed`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionUserMessageReceived
        if not isinstance(v, ActivityPayloadUnionUserMessageReceived):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionUserMessageReceived`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionFileAdded
        if not isinstance(v, ActivityPayloadUnionFileAdded):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionFileAdded`")
        else:
            match += 1
        # validate data type: ActivityPayloadUnionGraphUpdated
        if not isinstance(v, ActivityPayloadUnionGraphUpdated):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ActivityPayloadUnionGraphUpdated`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in AgentsExecutionActivityPayloadUnion with oneOf schemas: ActivityPayloadUnionActionCompleted, ActivityPayloadUnionActionFailed, ActivityPayloadUnionActionStarted, ActivityPayloadUnionFileAdded, ActivityPayloadUnionGeneric, ActivityPayloadUnionGraphUpdated, ActivityPayloadUnionStatusChanged, ActivityPayloadUnionStepCompleted, ActivityPayloadUnionStepStarted, ActivityPayloadUnionTerminal, ActivityPayloadUnionTransitionedNode, ActivityPayloadUnionUserMessageReceived. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in AgentsExecutionActivityPayloadUnion with oneOf schemas: ActivityPayloadUnionActionCompleted, ActivityPayloadUnionActionFailed, ActivityPayloadUnionActionStarted, ActivityPayloadUnionFileAdded, ActivityPayloadUnionGeneric, ActivityPayloadUnionGraphUpdated, ActivityPayloadUnionStatusChanged, ActivityPayloadUnionStepCompleted, ActivityPayloadUnionStepStarted, ActivityPayloadUnionTerminal, ActivityPayloadUnionTransitionedNode, ActivityPayloadUnionUserMessageReceived. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into ActivityPayloadUnionTerminal
        try:
            instance.actual_instance = ActivityPayloadUnionTerminal.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionGeneric
        try:
            instance.actual_instance = ActivityPayloadUnionGeneric.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionStepStarted
        try:
            instance.actual_instance = ActivityPayloadUnionStepStarted.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionStepCompleted
        try:
            instance.actual_instance = ActivityPayloadUnionStepCompleted.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionTransitionedNode
        try:
            instance.actual_instance = ActivityPayloadUnionTransitionedNode.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionStatusChanged
        try:
            instance.actual_instance = ActivityPayloadUnionStatusChanged.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionActionStarted
        try:
            instance.actual_instance = ActivityPayloadUnionActionStarted.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionActionCompleted
        try:
            instance.actual_instance = ActivityPayloadUnionActionCompleted.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionActionFailed
        try:
            instance.actual_instance = ActivityPayloadUnionActionFailed.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionUserMessageReceived
        try:
            instance.actual_instance = ActivityPayloadUnionUserMessageReceived.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionFileAdded
        try:
            instance.actual_instance = ActivityPayloadUnionFileAdded.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ActivityPayloadUnionGraphUpdated
        try:
            instance.actual_instance = ActivityPayloadUnionGraphUpdated.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into AgentsExecutionActivityPayloadUnion with oneOf schemas: ActivityPayloadUnionActionCompleted, ActivityPayloadUnionActionFailed, ActivityPayloadUnionActionStarted, ActivityPayloadUnionFileAdded, ActivityPayloadUnionGeneric, ActivityPayloadUnionGraphUpdated, ActivityPayloadUnionStatusChanged, ActivityPayloadUnionStepCompleted, ActivityPayloadUnionStepStarted, ActivityPayloadUnionTerminal, ActivityPayloadUnionTransitionedNode, ActivityPayloadUnionUserMessageReceived. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into AgentsExecutionActivityPayloadUnion with oneOf schemas: ActivityPayloadUnionActionCompleted, ActivityPayloadUnionActionFailed, ActivityPayloadUnionActionStarted, ActivityPayloadUnionFileAdded, ActivityPayloadUnionGeneric, ActivityPayloadUnionGraphUpdated, ActivityPayloadUnionStatusChanged, ActivityPayloadUnionStepCompleted, ActivityPayloadUnionStepStarted, ActivityPayloadUnionTerminal, ActivityPayloadUnionTransitionedNode, ActivityPayloadUnionUserMessageReceived. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], ActivityPayloadUnionActionCompleted, ActivityPayloadUnionActionFailed, ActivityPayloadUnionActionStarted, ActivityPayloadUnionFileAdded, ActivityPayloadUnionGeneric, ActivityPayloadUnionGraphUpdated, ActivityPayloadUnionStatusChanged, ActivityPayloadUnionStepCompleted, ActivityPayloadUnionStepStarted, ActivityPayloadUnionTerminal, ActivityPayloadUnionTransitionedNode, ActivityPayloadUnionUserMessageReceived]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())





# coding: utf-8

"""
    Asteroid Agents API

    Version 1 of the Asteroid Agents API

    The version of the OpenAPI document: v1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class Cookie(BaseModel):
    """
    Cookie
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, description="Unique identifier for the cookie")
    name: StrictStr = Field(description="Display name for the cookie")
    key: StrictStr = Field(description="The cookie name/key")
    value: StrictStr = Field(description="The cookie value")
    expiry: Optional[datetime] = Field(default=None, description="When the cookie expires (optional)")
    domain: StrictStr = Field(description="The domain for which the cookie is valid")
    secure: StrictBool = Field(description="Whether the cookie should only be sent over HTTPS")
    same_site: StrictStr = Field(description="SameSite attribute for the cookie")
    http_only: StrictBool = Field(description="Whether the cookie should be accessible only via HTTP(S)")
    created_at: datetime = Field(description="When the cookie was created")
    __properties: ClassVar[List[str]] = ["id", "name", "key", "value", "expiry", "domain", "secure", "same_site", "http_only", "created_at"]

    @field_validator('same_site')
    def same_site_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['Strict', 'Lax', 'None']):
            raise ValueError("must be one of enum values ('Strict', 'Lax', 'None')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Cookie from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Cookie from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "name": obj.get("name"),
            "key": obj.get("key"),
            "value": obj.get("value"),
            "expiry": obj.get("expiry"),
            "domain": obj.get("domain"),
            "secure": obj.get("secure"),
            "same_site": obj.get("same_site"),
            "http_only": obj.get("http_only"),
            "created_at": obj.get("created_at")
        })
        return _obj





# coding: utf-8

"""
    Agent Service

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field
from typing import Any, ClassVar, Dict, List, Optional
from asteroid_odyssey.agents_v2_gen.models.agents_execution_awaiting_confirmation_payload import AgentsExecutionAwaitingConfirmationPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_cancelled_payload import AgentsExecutionCancelledPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_completed_payload import AgentsExecutionCompletedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_failed_payload import AgentsExecutionFailedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_paused_payload import AgentsExecutionPausedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_status import AgentsExecutionStatus
from typing import Optional, Set
from typing_extensions import Self

class AgentsExecutionActivityStatusChangedPayload(BaseModel):
    """
    AgentsExecutionActivityStatusChangedPayload
    """ # noqa: E501
    awaiting_confirmation_payload: Optional[AgentsExecutionAwaitingConfirmationPayload] = Field(default=None, alias="awaitingConfirmationPayload")
    cancelled_payload: Optional[AgentsExecutionCancelledPayload] = Field(default=None, alias="cancelledPayload")
    completed_payload: Optional[AgentsExecutionCompletedPayload] = Field(default=None, alias="completedPayload")
    failed_payload: Optional[AgentsExecutionFailedPayload] = Field(default=None, alias="failedPayload")
    paused_payload: Optional[AgentsExecutionPausedPayload] = Field(default=None, alias="pausedPayload")
    status: AgentsExecutionStatus
    __properties: ClassVar[List[str]] = ["awaitingConfirmationPayload", "cancelledPayload", "completedPayload", "failedPayload", "pausedPayload", "status"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AgentsExecutionActivityStatusChangedPayload from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of awaiting_confirmation_payload
        if self.awaiting_confirmation_payload:
            _dict['awaitingConfirmationPayload'] = self.awaiting_confirmation_payload.to_dict()
        # override the default output from pydantic by calling `to_dict()` of cancelled_payload
        if self.cancelled_payload:
            _dict['cancelledPayload'] = self.cancelled_payload.to_dict()
        # override the default output from pydantic by calling `to_dict()` of completed_payload
        if self.completed_payload:
            _dict['completedPayload'] = self.completed_payload.to_dict()
        # override the default output from pydantic by calling `to_dict()` of failed_payload
        if self.failed_payload:
            _dict['failedPayload'] = self.failed_payload.to_dict()
        # override the default output from pydantic by calling `to_dict()` of paused_payload
        if self.paused_payload:
            _dict['pausedPayload'] = self.paused_payload.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AgentsExecutionActivityStatusChangedPayload from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "awaitingConfirmationPayload": AgentsExecutionAwaitingConfirmationPayload.from_dict(obj["awaitingConfirmationPayload"]) if obj.get("awaitingConfirmationPayload") is not None else None,
            "cancelledPayload": AgentsExecutionCancelledPayload.from_dict(obj["cancelledPayload"]) if obj.get("cancelledPayload") is not None else None,
            "completedPayload": AgentsExecutionCompletedPayload.from_dict(obj["completedPayload"]) if obj.get("completedPayload") is not None else None,
            "failedPayload": AgentsExecutionFailedPayload.from_dict(obj["failedPayload"]) if obj.get("failedPayload") is not None else None,
            "pausedPayload": AgentsExecutionPausedPayload.from_dict(obj["pausedPayload"]) if obj.get("pausedPayload") is not None else None,
            "status": obj.get("status")
        })
        return _obj





# flake8: noqa

# import apis into api package
from asteroid_odyssey.agents_v1_gen.api.api_api import APIApi
from asteroid_odyssey.agents_v1_gen.api.agent_profile_api import AgentProfileApi
from asteroid_odyssey.agents_v1_gen.api.execution_api import ExecutionApi




# coding: utf-8

# flake8: noqa
"""
    Agent Service

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_action_completed import ActivityPayloadUnionActionCompleted
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_action_failed import ActivityPayloadUnionActionFailed
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_action_started import ActivityPayloadUnionActionStarted
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_file_added import ActivityPayloadUnionFileAdded
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_generic import ActivityPayloadUnionGeneric
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_graph_updated import ActivityPayloadUnionGraphUpdated
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_status_changed import ActivityPayloadUnionStatusChanged
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_step_completed import ActivityPayloadUnionStepCompleted
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_step_started import ActivityPayloadUnionStepStarted
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_terminal import ActivityPayloadUnionTerminal
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_transitioned_node import ActivityPayloadUnionTransitionedNode
from asteroid_odyssey.agents_v2_gen.models.activity_payload_union_user_message_received import ActivityPayloadUnionUserMessageReceived
from asteroid_odyssey.agents_v2_gen.models.agent_list200_response import AgentList200Response
from asteroid_odyssey.agents_v2_gen.models.agents_agent_base import AgentsAgentBase
from asteroid_odyssey.agents_v2_gen.models.agents_agent_sort_field import AgentsAgentSortField
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity import AgentsExecutionActivity
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_action_completed_payload import AgentsExecutionActivityActionCompletedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_action_failed_payload import AgentsExecutionActivityActionFailedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_action_started_payload import AgentsExecutionActivityActionStartedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_file_added_payload import AgentsExecutionActivityFileAddedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_generic_payload import AgentsExecutionActivityGenericPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_graph_updated_payload import AgentsExecutionActivityGraphUpdatedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_payload_union import AgentsExecutionActivityPayloadUnion
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_status_changed_payload import AgentsExecutionActivityStatusChangedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_step_completed_payload import AgentsExecutionActivityStepCompletedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_step_started_payload import AgentsExecutionActivityStepStartedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_transitioned_node_payload import AgentsExecutionActivityTransitionedNodePayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_activity_user_message_received_payload import AgentsExecutionActivityUserMessageReceivedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_awaiting_confirmation_payload import AgentsExecutionAwaitingConfirmationPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_cancel_reason import AgentsExecutionCancelReason
from asteroid_odyssey.agents_v2_gen.models.agents_execution_cancelled_payload import AgentsExecutionCancelledPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_completed_payload import AgentsExecutionCompletedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_failed_payload import AgentsExecutionFailedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_graph_update import AgentsExecutionGraphUpdate
from asteroid_odyssey.agents_v2_gen.models.agents_execution_node_details import AgentsExecutionNodeDetails
from asteroid_odyssey.agents_v2_gen.models.agents_execution_paused_payload import AgentsExecutionPausedPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_status import AgentsExecutionStatus
from asteroid_odyssey.agents_v2_gen.models.agents_execution_terminal_payload import AgentsExecutionTerminalPayload
from asteroid_odyssey.agents_v2_gen.models.agents_execution_transition_details import AgentsExecutionTransitionDetails
from asteroid_odyssey.agents_v2_gen.models.agents_execution_update_type import AgentsExecutionUpdateType
from asteroid_odyssey.agents_v2_gen.models.agents_execution_user_messages_add_text_body import AgentsExecutionUserMessagesAddTextBody
from asteroid_odyssey.agents_v2_gen.models.agents_files_file import AgentsFilesFile
from asteroid_odyssey.agents_v2_gen.models.common_bad_request_error_body import CommonBadRequestErrorBody
from asteroid_odyssey.agents_v2_gen.models.common_error import CommonError
from asteroid_odyssey.agents_v2_gen.models.common_forbidden_error_body import CommonForbiddenErrorBody
from asteroid_odyssey.agents_v2_gen.models.common_not_found_error_body import CommonNotFoundErrorBody
from asteroid_odyssey.agents_v2_gen.models.common_sort_direction import CommonSortDirection
from asteroid_odyssey.agents_v2_gen.models.version import Version
